// WARNING: Portions of this file are autogenerated from codes.def and /builddir/build/BUILD/kdelibs-4.1.2/kjs/bytecode/machine.cpp.in.
// (which is what the licensing terms apply to)
// Any changes you make here may be lost!
#line 1 "/builddir/build/BUILD/kdelibs-4.1.2/kjs/bytecode/machine.cpp.in"
/*
 *  Main VM dispatch loop and related routines for KJS/Frostbyte
 *  This file is part of the KDE libraries
 *  Copyright (C) 2008 Maksim Orlovich (maksim@kde.org)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "completion.h"
#include "ExecState.h"
#include "value.h"

#include "nodes.h"
#include "opcodes.h"
#include "LocalStorage.h"
#include "bytecode/opargs.h"
#include "bytecode/machine.h"
#include "scriptfunction.h"
#include "internal.h" // for printInfo
#include "ustring.h"

#include <stdio.h>

namespace KJS {

// #define COUNT_INSTRS

// A little profiling aid -- counts the # of dynamic instances of each instruction.
#ifdef COUNT_INSTRS

static unsigned instrCounts[OpByteCode_NumValues];

struct InstrCountPrinter
{
    ~InstrCountPrinter() {
        for (int c = 0; c < OpByteCode_NumValues; ++c) {
            printf("%9d %s\n", instrCounts[c], OpByteCodeVals[c]);
        }
    }
};

static InstrCountPrinter dumpCounts;
#endif

// ### Need to consolidate this w/things remaining in nodes.cpp somehow.
static void substitute(UString &string, const UString &substring)
{
    int position = string.find("%s");
    assert(position != -1);
    UString newString = string.substr(0, position);
    newString.append(substring);
    newString.append(string.substr(position + 2));
    string = newString;
}

void throwUndefinedVariableError(ExecState* exec, const Identifier& ident)
{
    UString msg = "Can't find variable: %s";
    substitute(msg, ident.ustring());
    throwError(exec, ReferenceError, msg);
}

static JSValue* typeStringForValue(JSValue* v)
{
    switch (v->type()) {
    case UndefinedType:
        return jsString("undefined");
    case NullType:
        return jsString("object");
    case BooleanType:
        return jsString("boolean");
    case NumberType:
        return jsString("number");
    case StringType:
        return jsString("string");
    default:
        if (v->isObject()) {
            // Return "undefined" for objects that should be treated
            // as null when doing comparisons.
            if (static_cast<JSObject*>(v)->masqueradeAsUndefined())
                return jsString("undefined");
            else if (static_cast<JSObject*>(v)->isFunctionType())
                return jsString("function");
        }

        return jsString("object");
    }
}

template<bool errorOut, bool trySkipOne, bool replaceActivations>
ALWAYS_INLINE void lookupScopeAndFetch(ExecState* exec, Identifier* varName, JSValue*& scopeOut, JSValue*& valOut)
{
    const ScopeChain& chain = exec->scopeChain();
    ScopeChainIterator iter = chain.begin();
    ScopeChainIterator end = chain.end();

    // we must always have something in the scope chain
    assert(iter != end);

    JSObject *scopeObj = *iter;
    if (trySkipOne && !scopeObj->isLocalInjected()) {
        // Unless eval introduced new variables dynamically,
        // we know this isn't in the top scope
        ++iter;
    }

    PropertySlot slot;
    do {
        scopeObj = *iter;
        ++iter;
        if (scopeObj->getPropertySlot(exec, *varName, slot)) {
            if (!replaceActivations) {
                scopeOut = scopeObj;
            } else {
                // Common case: we found it in global object -- no need to check
                // if it's an activation
                if (iter == end || !scopeObj->isActivation())
                    scopeOut = scopeObj;
                else
                    scopeOut = exec->dynamicInterpreter()->globalObject();
            }
            valOut   = slot.getValue(exec, scopeObj, *varName);
            return;
        }
    } while (iter != end);

    scopeOut = scopeObj;
    valOut = jsUndefined();
    if (errorOut) {
        throwUndefinedVariableError(exec, *varName);
    }
}



ALWAYS_INLINE_INTO JSValue*
Machine::runBlock(ExecState* exec, const CodeBlock& codeBlock, ExecState* parentExec) {

    const unsigned char* base = codeBlock.data();
    const unsigned char* pc   = base;
    List workList;
    LocalStorageEntry* localStore = exec->localStorage();
    exec->setMachineRegisters(base, &pc, &localStore);
    JSObject* globalObject = exec->dynamicInterpreter()->globalObject();

    while (true) {
#ifdef COUNT_INSTRS
        OpByteCode opCodeLog = *reinterpret_cast<const OpByteCode*>(pc);
        ++instrCounts[opCodeLog];
#endif
#line 169 "machine.cpp"
        OpByteCode op = *reinterpret_cast<const OpByteCode*>(pc);
        switch (op) {
            case OpByteCode_RBool_Int32_Ireg_Ibool: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                bool in = reinterpret_cast<const NarrowArg*>(localPC -4)->boolVal;
#line 78 "codes.def"
                int32_t out = convertIBool_Int32(exec, in);
                localStore[fbDestReg].val.int32Val = out;
            }
            break;

            case OpByteCode_RBool_Int32_Ireg_Rbool: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                bool in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.boolVal;
#line 78 "codes.def"
                int32_t out = convertIBool_Int32(exec, in);
                localStore[fbDestReg].val.int32Val = out;
            }
            break;

            case OpByteCode_RBool_Value_Ireg_Ibool: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                bool in = reinterpret_cast<const NarrowArg*>(localPC -4)->boolVal;
#line 87 "codes.def"
                JSValue* out = convertIBool_Value(exec, in);
                localStore[fbDestReg].val.valueVal = out;
            }
            break;

            case OpByteCode_RBool_Value_Ireg_Rbool: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                bool in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.boolVal;
#line 87 "codes.def"
                JSValue* out = convertIBool_Value(exec, in);
                localStore[fbDestReg].val.valueVal = out;
            }
            break;

            case OpByteCode_RBool_Number_Ireg_Ibool: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                bool in = reinterpret_cast<const NarrowArg*>(localPC -4)->boolVal;
#line 96 "codes.def"
                double out = convertIBool_Number(exec, in);
                localStore[fbDestReg].val.numberVal = out;
            }
            break;

            case OpByteCode_RBool_Number_Ireg_Rbool: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                bool in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.boolVal;
#line 96 "codes.def"
                double out = convertIBool_Number(exec, in);
                localStore[fbDestReg].val.numberVal = out;
            }
            break;

            case OpByteCode_RValue_Bool_Ireg_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RValue_Bool_Ireg_Ivalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* in = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 108 "codes.def"
                bool out = convertIValue_Bool(exec, in);
                localStore[fbDestReg].val.boolVal = out;
            }
            break;

            case OpByteCode_RValue_Bool_Ireg_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 108 "codes.def"
                bool out = convertIValue_Bool(exec, in);
                localStore[fbDestReg].val.boolVal = out;
            }
            break;

            case OpByteCode_RValue_Int32_Ireg_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RValue_Int32_Ireg_Ivalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* in = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 117 "codes.def"
                int32_t out = convertIValue_Int32(exec, in);
                localStore[fbDestReg].val.int32Val = out;
            }
            break;

            case OpByteCode_RValue_Int32_Ireg_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 117 "codes.def"
                int32_t out = convertIValue_Int32(exec, in);
                localStore[fbDestReg].val.int32Val = out;
            }
            break;

            case OpByteCode_RValue_Number_Ireg_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RValue_Number_Ireg_Ivalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* in = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 126 "codes.def"
                double out = convertIValue_Number(exec, in);
                localStore[fbDestReg].val.numberVal = out;
            }
            break;

            case OpByteCode_RValue_Number_Ireg_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 126 "codes.def"
                double out = convertIValue_Number(exec, in);
                localStore[fbDestReg].val.numberVal = out;
            }
            break;

            case OpByteCode_RInt32_Bool_Ireg_Iint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t in = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 138 "codes.def"
                bool out = convertIInt32_Bool(exec, in);
                localStore[fbDestReg].val.boolVal = out;
            }
            break;

            case OpByteCode_RInt32_Bool_Ireg_Rint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 138 "codes.def"
                bool out = convertIInt32_Bool(exec, in);
                localStore[fbDestReg].val.boolVal = out;
            }
            break;

            case OpByteCode_RInt32_Value_Ireg_Iint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t in = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 147 "codes.def"
                JSValue* out = convertIInt32_Value(exec, in);
                localStore[fbDestReg].val.valueVal = out;
            }
            break;

            case OpByteCode_RInt32_Value_Ireg_Rint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 147 "codes.def"
                JSValue* out = convertIInt32_Value(exec, in);
                localStore[fbDestReg].val.valueVal = out;
            }
            break;

            case OpByteCode_RInt32_Number_Ireg_Iint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t in = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 156 "codes.def"
                double out = convertIInt32_Number(exec, in);
                localStore[fbDestReg].val.numberVal = out;
            }
            break;

            case OpByteCode_RInt32_Number_Ireg_Rint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 156 "codes.def"
                double out = convertIInt32_Number(exec, in);
                localStore[fbDestReg].val.numberVal = out;
            }
            break;

            case OpByteCode_RNumber_Bool_Ireg_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RNumber_Bool_Ireg_Inumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double in = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 168 "codes.def"
                bool out = convertINumber_Bool(exec, in);
                localStore[fbDestReg].val.boolVal = out;
            }
            break;

            case OpByteCode_RNumber_Bool_Ireg_Rnumber: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 168 "codes.def"
                bool out = convertINumber_Bool(exec, in);
                localStore[fbDestReg].val.boolVal = out;
            }
            break;

            case OpByteCode_RNumber_Value_Ireg_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RNumber_Value_Ireg_Inumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double in = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 177 "codes.def"
                JSValue* out = convertINumber_Value(exec, in);
                localStore[fbDestReg].val.valueVal = out;
            }
            break;

            case OpByteCode_RNumber_Value_Ireg_Rnumber: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 177 "codes.def"
                JSValue* out = convertINumber_Value(exec, in);
                localStore[fbDestReg].val.valueVal = out;
            }
            break;

            case OpByteCode_RNumber_Int32_Ireg_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RNumber_Int32_Ireg_Inumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double in = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 186 "codes.def"
                int32_t out = convertINumber_Int32(exec, in);
                localStore[fbDestReg].val.int32Val = out;
            }
            break;

            case OpByteCode_RNumber_Int32_Ireg_Rnumber: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 186 "codes.def"
                int32_t out = convertINumber_Int32(exec, in);
                localStore[fbDestReg].val.int32Val = out;
            }
            break;

            case OpByteCode_RInt32_Value_NonImm_Ireg_Iint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t val = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 200 "codes.def"
                localStore[fbDestReg].val.valueVal = jsNumberCell(val);
            }
            break;

            case OpByteCode_RInt32_Value_NonImm_Ireg_Rint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 200 "codes.def"
                localStore[fbDestReg].val.valueVal = jsNumberCell(val);
            }
            break;

            case OpByteCode_RNum_Value_NonImm_Ireg_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RNum_Value_NonImm_Ireg_Inumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double val = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 206 "codes.def"
                localStore[fbDestReg].val.valueVal = jsNumberCell(val);
            }
            break;

            case OpByteCode_RNum_Value_NonImm_Ireg_Rnumber: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 206 "codes.def"
                localStore[fbDestReg].val.valueVal = jsNumberCell(val);
            }
            break;

            case OpByteCode_PrintInfo_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_PrintInfo_Ivalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                JSValue* in = reinterpret_cast<const WideArg*>(localPC -8)->valueVal;
#line 221 "codes.def"
                printInfo(exec, "", in);
            }
            break;

            case OpByteCode_PrintInfo_Rvalue: {
                pc += 8;
                const unsigned char* localPC = pc;
                JSValue* in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 221 "codes.def"
                printInfo(exec, "", in);
            }
            break;

            case OpByteCode_AtStatement_Inode_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_AtStatement_Inode: {
                pc += 12;
                const unsigned char* localPC = pc;
                Node* in = reinterpret_cast<const WideArg*>(localPC -8)->nodeVal;
#line 230 "codes.def"
                static_cast<StatementNode*>(in)->hitStatement(exec);
            }
            break;

            case OpByteCode_Exit: {
                pc += 4;
                const unsigned char* localPC = pc;
#line 239 "codes.def"
                return jsUndefined();
            }
            break;

            case OpByteCode_ExitEval_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_ExitEval_Ivalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                JSValue* val = reinterpret_cast<const WideArg*>(localPC -8)->valueVal;
#line 245 "codes.def"
                return val;
            }
            break;

            case OpByteCode_ExitEval_Rvalue: {
                pc += 8;
                const unsigned char* localPC = pc;
                JSValue* val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 245 "codes.def"
                return val;
            }
            break;

            case OpByteCode_Jump_Iaddr: {
                pc += 8;
                const unsigned char* localPC = pc;
                Addr dest = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 251 "codes.def"
                pc = base + dest;
            }
            break;

            case OpByteCode_IfJump_Ibool_Iaddr: {
                pc += 12;
                const unsigned char* localPC = pc;
                bool condResult = reinterpret_cast<const NarrowArg*>(localPC -8)->boolVal;
                Addr dest = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 257 "codes.def"
                if (condResult)
                    pc = base + dest;
            }
            break;

            case OpByteCode_IfJump_Rbool_Iaddr: {
                pc += 12;
                const unsigned char* localPC = pc;
                bool condResult = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.boolVal;
                Addr dest = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 257 "codes.def"
                if (condResult)
                    pc = base + dest;
            }
            break;

            case OpByteCode_IfJump_Rvalue_Iaddr: {
                pc += 12;
                const unsigned char* localPC = pc;
                bool condResult = convertIValue_Bool(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal);
                Addr dest = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 257 "codes.def"
                if (condResult)
                    pc = base + dest;
            }
            break;

            case OpByteCode_IfNotJump_Ibool_Iaddr: {
                pc += 12;
                const unsigned char* localPC = pc;
                bool condResult = reinterpret_cast<const NarrowArg*>(localPC -8)->boolVal;
                Addr dest = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 266 "codes.def"
                if (!condResult)
                    pc = base +dest;
            }
            break;

            case OpByteCode_IfNotJump_Rbool_Iaddr: {
                pc += 12;
                const unsigned char* localPC = pc;
                bool condResult = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.boolVal;
                Addr dest = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 266 "codes.def"
                if (!condResult)
                    pc = base +dest;
            }
            break;

            case OpByteCode_IfNotJump_Rvalue_Iaddr: {
                pc += 12;
                const unsigned char* localPC = pc;
                bool condResult = convertIValue_Bool(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal);
                Addr dest = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 266 "codes.def"
                if (!condResult)
                    pc = base +dest;
            }
            break;

            case OpByteCode_BeginForIn_Ireg_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BeginForIn_Ireg_Ivalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* e = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 277 "codes.def"
                exec->pushPropertyNameArray();

                if (!e->isUndefinedOrNull()) {
                    JSObject* v = e->toObject(exec);
                    // The above might raise an exception, which would cleanup the array!
                    if (pc != localPC) break;

                    v->getPropertyNames(exec, exec->activePropertyNameArray());
                    localStore[fbDestReg].val.valueVal = v;
                } else {
                    // The array is empty here, so this will never be accessed
                    localStore[fbDestReg].val.valueVal = jsUndefined();
                }
            }
            break;

            case OpByteCode_BeginForIn_Ireg_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* e = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 277 "codes.def"
                exec->pushPropertyNameArray();

                if (!e->isUndefinedOrNull()) {
                    JSObject* v = e->toObject(exec);
                    // The above might raise an exception, which would cleanup the array!
                    if (pc != localPC) break;

                    v->getPropertyNames(exec, exec->activePropertyNameArray());
                    localStore[fbDestReg].val.valueVal = v;
                } else {
                    // The array is empty here, so this will never be accessed
                    localStore[fbDestReg].val.valueVal = jsUndefined();
                }
            }
            break;

            case OpByteCode_EndForIn: {
                pc += 4;
                const unsigned char* localPC = pc;
#line 296 "codes.def"
                exec->popPropertyNameArray();
            }
            break;

            case OpByteCode_NextForInEntry_Ireg_Ivalue_Iaddr_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NextForInEntry_Ireg_Ivalue_Iaddr: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* e = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                Addr jumpToIfDone = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 302 "codes.def"
                PropertyNameArray& pa  = exec->activePropertyNameArray();

                // Invariant: pos = next entry to consider.
                int& pos = exec->activePropertyNameIter();

                localStore[fbDestReg].val.valueVal = jsUndefined();
                while (pos < pa.size()) {
                    ASSERT(e->isObject());
                    JSObject* v = static_cast<JSObject*>(e);

                    Identifier& name = pa[pos];
                    ++pos;

                    if (v->hasProperty(exec, name)) {
                        // Wasn't deleted during iteration..
                        localStore[fbDestReg].val.valueVal = jsOwnedString(name.ustring());
                        break;
                    }
                }

                if (localStore[fbDestReg].val.valueVal == jsUndefined()) // Nothing found. Jump to end of the loop
                    pc = base + jumpToIfDone;
            }
            break;

            case OpByteCode_NextForInEntry_Ireg_Rvalue_Iaddr: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* e = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                Addr jumpToIfDone = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 302 "codes.def"
                PropertyNameArray& pa  = exec->activePropertyNameArray();

                // Invariant: pos = next entry to consider.
                int& pos = exec->activePropertyNameIter();

                localStore[fbDestReg].val.valueVal = jsUndefined();
                while (pos < pa.size()) {
                    ASSERT(e->isObject());
                    JSObject* v = static_cast<JSObject*>(e);

                    Identifier& name = pa[pos];
                    ++pos;

                    if (v->hasProperty(exec, name)) {
                        // Wasn't deleted during iteration..
                        localStore[fbDestReg].val.valueVal = jsOwnedString(name.ustring());
                        break;
                    }
                }

                if (localStore[fbDestReg].val.valueVal == jsUndefined()) // Nothing found. Jump to end of the loop
                    pc = base + jumpToIfDone;
            }
            break;

            case OpByteCode_PushScope_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_PushScope_Ivalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                JSValue* in = reinterpret_cast<const WideArg*>(localPC -8)->valueVal;
#line 331 "codes.def"
                JSObject* obj = in->toObject(exec);
                if (exec->hadException())
                    continue; // will jump to handler.

                exec->pushScope(obj);
                exec->pushExceptionHandler(ExecState::PopScope);
            }
            break;

            case OpByteCode_PushScope_Rvalue: {
                pc += 8;
                const unsigned char* localPC = pc;
                JSValue* in = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 331 "codes.def"
                JSObject* obj = in->toObject(exec);
                if (exec->hadException())
                    continue; // will jump to handler.

                exec->pushScope(obj);
                exec->pushExceptionHandler(ExecState::PopScope);
            }
            break;

            case OpByteCode_UnwindStacks_Iint32: {
                pc += 8;
                const unsigned char* localPC = pc;
                int32_t handlers = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 342 "codes.def"
                // This op is used when a break or continue is used to jump out
                // of a context that updated various stacks in ExecState,
                // in order to clean them up. Each of those ops contributes an
                // entry to the main exception handler stack, and perhaps some others.
                // We unwind the main stack, using the cleanup entries on it as appropriate.
                exec->quietUnwind(handlers);
            }
            break;

            case OpByteCode_UnwindStacks_Rint32: {
                pc += 8;
                const unsigned char* localPC = pc;
                int32_t handlers = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 342 "codes.def"
                // This op is used when a break or continue is used to jump out
                // of a context that updated various stacks in ExecState,
                // in order to clean them up. Each of those ops contributes an
                // entry to the main exception handler stack, and perhaps some others.
                // We unwind the main stack, using the cleanup entries on it as appropriate.
                exec->quietUnwind(handlers);
            }
            break;

            case OpByteCode_PopScope: {
                pc += 4;
                const unsigned char* localPC = pc;
#line 353 "codes.def"
                exec->popScope();
                exec->popExceptionHandler();
            }
            break;

            case OpByteCode_PopExceptionHandler: {
                pc += 4;
                const unsigned char* localPC = pc;
#line 360 "codes.def"
                exec->popExceptionHandler();
            }
            break;

            case OpByteCode_PushExceptionHandler_Iaddr: {
                pc += 8;
                const unsigned char* localPC = pc;
                Addr handler = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 366 "codes.def"
                exec->pushExceptionHandler(ExecState::JumpToCatch, handler);
            }
            break;

            case OpByteCode_EnterCatch_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_EnterCatch_Iident: {
                pc += 12;
                const unsigned char* localPC = pc;
                Identifier* exceptionIdent = reinterpret_cast<const WideArg*>(localPC -8)->identVal;
#line 372 "codes.def"
                // First, check if we have a non-exception indirect continuation,
                // if so, we re-raise it directly to the finally (and skip the rest of the catch section)
                if (exec->abruptCompletion().complType() != Throw) {
                    // By this point, the previous handler has been popped, so we should have the
                    // 'finally' bound as the handler. Hence, set the completion again to jump to it.
                    exec->setAbruptCompletion(exec->abruptCompletion());
                    break;
                }

                // Now we have a plain, old exception. Grab & clear exception it exec state,
                // and put it in a scope w/a name to it
                JSValue* exception = exec->exception();
                exec->clearException();

                JSObject *obj = new JSObject;
                obj->put(exec, *exceptionIdent, exception, DontDelete);
                exec->pushScope(obj);
                exec->pushExceptionHandler(ExecState::PopScope);
            }
            break;

            case OpByteCode_ExitCatch: {
                pc += 4;
                const unsigned char* localPC = pc;
#line 395 "codes.def"
                exec->popScope();
                exec->popExceptionHandler();
            }
            break;

            case OpByteCode_DeferCompletion: {
                pc += 4;
                const unsigned char* localPC = pc;
#line 402 "codes.def"
                exec->deferCompletion();
            }
            break;

            case OpByteCode_ReactivateCompletion_Ibool: {
                pc += 8;
                const unsigned char* localPC = pc;
                bool insideFinally = reinterpret_cast<const NarrowArg*>(localPC -4)->boolVal;
#line 408 "codes.def"
                JSValue* retVal = exec->reactivateCompletion(insideFinally);
                if (retVal)
                    return retVal;
            }
            break;

            case OpByteCode_ReactivateCompletion_Rbool: {
                pc += 8;
                const unsigned char* localPC = pc;
                bool insideFinally = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.boolVal;
#line 408 "codes.def"
                JSValue* retVal = exec->reactivateCompletion(insideFinally);
                if (retVal)
                    return retVal;
            }
            break;

            case OpByteCode_Throw_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Throw_Ivalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                JSValue* exception = reinterpret_cast<const WideArg*>(localPC -8)->valueVal;
#line 416 "codes.def"
                exec->setException(exception);
            }
            break;

            case OpByteCode_Throw_Rvalue: {
                pc += 8;
                const unsigned char* localPC = pc;
                JSValue* exception = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 416 "codes.def"
                exec->setException(exception);
            }
            break;

            case OpByteCode_ContBreakInTryFinally_Iaddr: {
                pc += 8;
                const unsigned char* localPC = pc;
                Addr dest = reinterpret_cast<const NarrowArg*>(localPC -4)->addrVal;
#line 422 "codes.def"
                exec->setAbruptCompletion(Completion(Continue /*or break, doesn't matter*/, 0, dest));
            }
            break;

            case OpByteCode_ReturnInTryFinally_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_ReturnInTryFinally_Ivalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                JSValue* val = reinterpret_cast<const WideArg*>(localPC -8)->valueVal;
#line 428 "codes.def"
                exec->setAbruptCompletion(Completion(ReturnValue, val));
            }
            break;

            case OpByteCode_ReturnInTryFinally_Rvalue: {
                pc += 8;
                const unsigned char* localPC = pc;
                JSValue* val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 428 "codes.def"
                exec->setAbruptCompletion(Completion(ReturnValue, val));
            }
            break;

            case OpByteCode_PropagateException: {
                pc += 4;
                const unsigned char* localPC = pc;
#line 434 "codes.def"
                JSValue* e = exec->exception();
                if (parentExec)
                    parentExec->setException(e);
                return e;
            }
            break;

            case OpByteCode_Return_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Return_Ivalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                JSValue* retVal = reinterpret_cast<const WideArg*>(localPC -8)->valueVal;
#line 443 "codes.def"
                return retVal;
            }
            break;

            case OpByteCode_Return_Rvalue: {
                pc += 8;
                const unsigned char* localPC = pc;
                JSValue* retVal = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 443 "codes.def"
                return retVal;
            }
            break;

            case OpByteCode_RaiseError_Inode_Iint32_Icstr_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RaiseError_Inode_Iint32_Icstr: {
                pc += 24;
                const unsigned char* localPC = pc;
                Node* node = reinterpret_cast<const WideArg*>(localPC -20)->nodeVal;
                int32_t errorType = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
                const char* msg = reinterpret_cast<const WideArg*>(localPC -12)->cstrVal;
#line 449 "codes.def"
                node->throwError(exec, ErrorType(errorType), msg);
            }
            break;

            case OpByteCode_RaiseError_Inode_Rint32_Icstr_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RaiseError_Inode_Rint32_Icstr: {
                pc += 24;
                const unsigned char* localPC = pc;
                Node* node = reinterpret_cast<const WideArg*>(localPC -20)->nodeVal;
                int32_t errorType = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
                const char* msg = reinterpret_cast<const WideArg*>(localPC -12)->cstrVal;
#line 449 "codes.def"
                node->throwError(exec, ErrorType(errorType), msg);
            }
            break;

            case OpByteCode_This_Ireg: {
                pc += 8;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
#line 457 "codes.def"
                localStore[fbDestReg].val.valueVal = exec->thisValue();
            }
            break;

            case OpByteCode_GetVariableObject_Ireg: {
                pc += 8;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
#line 463 "codes.def"
                localStore[fbDestReg].val.valueVal = exec->variableObject();
            }
            break;

            case OpByteCode_GetGlobalObject_Ireg: {
                pc += 8;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
#line 469 "codes.def"
                localStore[fbDestReg].val.valueVal = globalObject;
            }
            break;

            case OpByteCode_FunctionLookupAndGet_Ireg_Ireg_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_FunctionLookupAndGet_Ireg_Ireg_Iident: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                Register thisOutReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
#line 475 "codes.def"
                JSValue* res;
                JSValue* thisOut;
                lookupScopeAndFetch<true /*error out*/, false /* don't skip one*/, true /* want this value*/>
                        (exec, varName, thisOut, res);
                localStore[thisOutReg].val.valueVal = thisOut;
                localStore[fbDestReg].val.valueVal = res;
            }
            break;

            case OpByteCode_NonLocalFunctionLookupAndGet_Ireg_Ireg_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NonLocalFunctionLookupAndGet_Ireg_Ireg_Iident: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                Register thisOutReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
#line 486 "codes.def"
                JSValue* res;
                JSValue* thisOut;
                lookupScopeAndFetch<true /*error out*/, true /* skip one*/, true /* want this value*/>
                        (exec, varName, thisOut, res);
                localStore[fbDestReg].val.valueVal = res;
                localStore[thisOutReg].val.valueVal = thisOut;
            }
            break;

            case OpByteCode_ScopeLookupAndGetChecked_Ireg_Ireg_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_ScopeLookupAndGetChecked_Ireg_Ireg_Iident: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                Register valOutReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
#line 497 "codes.def"
                JSValue* scopeOut;
                JSValue* val;

                lookupScopeAndFetch<true /*error out*/, false /* don't skip one*/, false /* scope, not this*/>
                            (exec, varName, scopeOut, val);
                localStore[fbDestReg].val.valueVal = scopeOut;
                localStore[valOutReg].val.valueVal = val;
            }
            break;

            case OpByteCode_ScopeLookupAndGet_Ireg_Ireg_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_ScopeLookupAndGet_Ireg_Ireg_Iident: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                Register valOutReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
#line 509 "codes.def"
                JSValue* scopeOut;
                JSValue* val;

                lookupScopeAndFetch<false /*don't error out*/, false /* no skip*/, false /* scope, not this*/ >
                            (exec, varName, scopeOut, val);
                localStore[fbDestReg].val.valueVal = scopeOut;
                localStore[valOutReg].val.valueVal = val;
            }
            break;

            case OpByteCode_NonLocalScopeLookupAndGetChecked_Ireg_Ireg_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NonLocalScopeLookupAndGetChecked_Ireg_Ireg_Iident: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                Register valOutReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
#line 521 "codes.def"
                JSValue* scopeOut;
                JSValue* val;

                lookupScopeAndFetch<true /*error out*/, true /* skip one */, false /* scope, not this*/ >
                            (exec, varName, scopeOut, val);
                localStore[fbDestReg].val.valueVal = scopeOut;
                localStore[valOutReg].val.valueVal = val;
            }
            break;

            case OpByteCode_NonLocalScopeLookupAndGet_Ireg_Ireg_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NonLocalScopeLookupAndGet_Ireg_Ireg_Iident: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                Register valOutReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
#line 533 "codes.def"
                JSValue* scopeOut;
                JSValue* val;

                lookupScopeAndFetch<false /*don't error out*/, true /* skip one */, false /* scope, not this*/ >
                            (exec, varName, scopeOut, val);
                localStore[fbDestReg].val.valueVal = scopeOut;
                localStore[valOutReg].val.valueVal = val;
            }
            break;

            case OpByteCode_ScopeLookup_Ireg_Iident_Inode_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_ScopeLookup_Ireg_Iident_Inode: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -20)->identVal;
                Node* errorNode = reinterpret_cast<const WideArg*>(localPC -12)->nodeVal;
#line 548 "codes.def"
                const ScopeChain& chain = exec->scopeChain();
                ScopeChainIterator iter = chain.begin();
                ScopeChainIterator end = chain.end();

                // we must always have something in the scope chain
                assert(iter != end);

                JSObject *scopeObj;
                PropertySlot slot;
                do {
                    scopeObj = *iter;

                    if (scopeObj->getPropertySlot(exec, *varName, slot))
                        break;
                    ++iter;
                } while (iter != end);

                if (errorNode && iter == end) {
                    localStore[fbDestReg].val.valueVal = errorNode->throwUndefinedVariableError(exec, *varName);
                    break;
                }

                localStore[fbDestReg].val.valueVal = scopeObj;
            }
            break;

            case OpByteCode_NonLocalScopeLookup_Ireg_Iident_Inode_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NonLocalScopeLookup_Ireg_Iident_Inode: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -20)->identVal;
                Node* errorNode = reinterpret_cast<const WideArg*>(localPC -12)->nodeVal;
#line 577 "codes.def"
                const ScopeChain& chain = exec->scopeChain();
                ScopeChainIterator iter = chain.begin();
                ScopeChainIterator end = chain.end();

                // we must always have something in the scope chain
                assert(iter != end);

                JSObject *scopeObj = *iter;
                if (!scopeObj->isLocalInjected()) {
                    // Unless eval introduced new variables dynamically,
                    // we know this isn't in the top scope
                    ++iter;
                }

                PropertySlot slot;
                do {
                    scopeObj = *iter;

                    if (scopeObj->getPropertySlot(exec, *varName, slot))
                        break;
                    ++iter;
                } while (iter != end);

                if (errorNode && iter == end) {
                    localStore[fbDestReg].val.valueVal = errorNode->throwUndefinedVariableError(exec, *varName);
                    break;
                }
                localStore[fbDestReg].val.valueVal = scopeObj;
            }
            break;

            case OpByteCode_VarGet_Ireg_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_VarGet_Ireg_Iident: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -12)->identVal;
#line 612 "codes.def"
                JSValue* scopeOut;
                JSValue* val;

                lookupScopeAndFetch<true /*error out*/, false /* don't skip one*/, false /*no this*/ >(exec, varName, scopeOut, val);
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_NonLocalVarGet_Ireg_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NonLocalVarGet_Ireg_Iident: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -12)->identVal;
#line 623 "codes.def"
                JSValue* scopeOut;
                JSValue* val;

                lookupScopeAndFetch<true /*error out*/, true /* don't skip one*/, false /*no this*/ >(exec, varName, scopeOut, val);
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_RegPutValue_Ireg_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RegPutValue_Ireg_Ivalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register destReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* val = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 633 "codes.def"
                localStore[destReg].val.valueVal = val;
            }
            break;

            case OpByteCode_RegPutValue_Ireg_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register destReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 633 "codes.def"
                localStore[destReg].val.valueVal = val;
            }
            break;

            case OpByteCode_RegPutValue_Ireg_Rnumber: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register destReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* val = convertINumber_Value(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
#line 633 "codes.def"
                localStore[destReg].val.valueVal = val;
            }
            break;

            case OpByteCode_RegPutNumber_Ireg_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_RegPutNumber_Ireg_Inumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register destReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double val = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 641 "codes.def"
                localStore[destReg].val.numberVal = val;
            }
            break;

            case OpByteCode_RegPutNumber_Ireg_Rnumber: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register destReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 641 "codes.def"
                localStore[destReg].val.numberVal = val;
            }
            break;

            case OpByteCode_RegPutBool_Ireg_Ibool: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register destReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                bool val = reinterpret_cast<const NarrowArg*>(localPC -4)->boolVal;
#line 647 "codes.def"
                localStore[destReg].val.boolVal = val;
            }
            break;

            case OpByteCode_RegPutBool_Ireg_Rbool: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register destReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                bool val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.boolVal;
#line 647 "codes.def"
                localStore[destReg].val.boolVal = val;
            }
            break;

            case OpByteCode_RegPutInt32_Ireg_Iint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register destReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t val = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 653 "codes.def"
                localStore[destReg].val.int32Val = val;
            }
            break;

            case OpByteCode_RegPutInt32_Ireg_Rint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register destReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 653 "codes.def"
                localStore[destReg].val.int32Val = val;
            }
            break;

            case OpByteCode_ToObject_Ireg_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_ToObject_Ireg_Ivalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* val = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 659 "codes.def"
                localStore[fbDestReg].val.valueVal = val->toObject(exec);
            }
            break;

            case OpByteCode_ToObject_Ireg_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 659 "codes.def"
                localStore[fbDestReg].val.valueVal = val->toObject(exec);
            }
            break;

            case OpByteCode_SymPutKnownObject_Rvalue_Iident_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymPutKnownObject_Rvalue_Iident_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                JSValue* scope = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -20)->identVal;
                JSValue* val = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 666 "codes.def"
                ASSERT(scope->isObject());
                static_cast<JSObject*>(scope)->put(exec, *varName, val);
            }
            break;

            case OpByteCode_SymPutKnownObject_Rvalue_Iident_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymPutKnownObject_Rvalue_Iident_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* scope = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
                JSValue* val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 666 "codes.def"
                ASSERT(scope->isObject());
                static_cast<JSObject*>(scope)->put(exec, *varName, val);
            }
            break;

            case OpByteCode_SymPutKnownObject_Rvalue_Iident_Rnumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymPutKnownObject_Rvalue_Iident_Rnumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* scope = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
                JSValue* val = convertINumber_Value(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
#line 666 "codes.def"
                ASSERT(scope->isObject());
                static_cast<JSObject*>(scope)->put(exec, *varName, val);
            }
            break;

            case OpByteCode_SymGet_Ireg_Ivalue_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymGet_Ireg_Ivalue_Iident: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* base = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                Identifier* prop = reinterpret_cast<const WideArg*>(localPC -12)->identVal;
#line 676 "codes.def"
                JSObject* baseObj = base->toObject(exec);
                // toObject may fail and return an exception object, but get from it is harmless

                // IMPORTANT: The call and store to localStore[fbDestReg].val.valueVal should be two steps, since ::get can tear off frames!
                JSValue* val = baseObj->get(exec, *prop);
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_SymGet_Ireg_Rvalue_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymGet_Ireg_Rvalue_Iident: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                Identifier* prop = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
#line 676 "codes.def"
                JSObject* baseObj = base->toObject(exec);
                // toObject may fail and return an exception object, but get from it is harmless

                // IMPORTANT: The call and store to localStore[fbDestReg].val.valueVal should be two steps, since ::get can tear off frames!
                JSValue* val = baseObj->get(exec, *prop);
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_SymGetKnownObject_Ireg_Ivalue_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymGetKnownObject_Ireg_Ivalue_Iident: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* base = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                Identifier* prop = reinterpret_cast<const WideArg*>(localPC -12)->identVal;
#line 688 "codes.def"
                ASSERT(base->isObject());

                localStore[fbDestReg].val.valueVal = static_cast<JSObject*>(base)->get(exec, *prop);
            }
            break;

            case OpByteCode_SymGetKnownObject_Ireg_Rvalue_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymGetKnownObject_Ireg_Rvalue_Iident: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                Identifier* prop = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
#line 688 "codes.def"
                ASSERT(base->isObject());

                localStore[fbDestReg].val.valueVal = static_cast<JSObject*>(base)->get(exec, *prop);
            }
            break;

            case OpByteCode_SymGetAndBind_Ireg_Ireg_Rvalue_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymGetAndBind_Ireg_Ireg_Rvalue_Iident: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                Register objDest = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -20)->identVal;
#line 697 "codes.def"
                JSObject* baseObj = base->toObject(exec);
                // toObject may fail and return an exception object, but get from it is harmless
                localStore[objDest].val.valueVal = baseObj;

                // IMPORTANT: The call and store to localStore[fbDestReg].val.valueVal should be two steps, since ::get can tear off frames!
                JSValue* val = baseObj->get(exec, *varName);
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_BracketGet_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BracketGet_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 709 "codes.def"
                uint32_t i;

                if (v2->getUInt32(i)) {
                localStore[fbDestReg].val.valueVal = v1->getByIndex(exec, i);
                break;
                }

                JSObject *o = v1->toObject(exec);
                // Make sure to  handle exception immediately, here, since toString might throw, too!
                if (exec->hadException())
                    break;

                // IMPORTANT: The call and store to localStore[fbDestReg].val.valueVal should be two steps, since ::get can tear off frames!
                JSValue* val = o->get(exec, Identifier(v2->toString(exec)));
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_BracketGet_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 709 "codes.def"
                uint32_t i;

                if (v2->getUInt32(i)) {
                localStore[fbDestReg].val.valueVal = v1->getByIndex(exec, i);
                break;
                }

                JSObject *o = v1->toObject(exec);
                // Make sure to  handle exception immediately, here, since toString might throw, too!
                if (exec->hadException())
                    break;

                // IMPORTANT: The call and store to localStore[fbDestReg].val.valueVal should be two steps, since ::get can tear off frames!
                JSValue* val = o->get(exec, Identifier(v2->toString(exec)));
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_BracketGet_Ireg_Rvalue_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t prop = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 727 "codes.def"
                localStore[fbDestReg].val.valueVal = base->getByIndex(exec, prop);
            }
            break;

            case OpByteCode_BracketGet_Ireg_Rvalue_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t prop = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 727 "codes.def"
                localStore[fbDestReg].val.valueVal = base->getByIndex(exec, prop);
            }
            break;

            case OpByteCode_BracketGetAndBind_Ireg_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BracketGetAndBind_Ireg_Ireg_Rvalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                Register objDest = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
#line 734 "codes.def"
                uint32_t i;
                JSObject *o = v1->toObject(exec);
                // Make sure to  handle exception immediately, here, since toString might throw, too!
                if (exec->hadException())
                    break;

                localStore[objDest].val.valueVal = o;

                JSValue* val;
                if (v2->getUInt32(i))
                    val = o->get(exec, i);
                else
                    val = o->get(exec, Identifier(v2->toString(exec)));
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_BracketGetAndBind_Ireg_Ireg_Rvalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -16)->regVal;
                Register objDest = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 734 "codes.def"
                uint32_t i;
                JSObject *o = v1->toObject(exec);
                // Make sure to  handle exception immediately, here, since toString might throw, too!
                if (exec->hadException())
                    break;

                localStore[objDest].val.valueVal = o;

                JSValue* val;
                if (v2->getUInt32(i))
                    val = o->get(exec, i);
                else
                    val = o->get(exec, Identifier(v2->toString(exec)));
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_BracketGetAndBind_Ireg_Ireg_Rvalue_Iint32: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -16)->regVal;
                Register objDest = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t prop = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 751 "codes.def"
                JSObject* baseObj = base->toObject(exec);
                // toObject may fail and return an exception object, but get from it is harmless
                localStore[objDest].val.valueVal = baseObj;

                localStore[fbDestReg].val.valueVal = baseObj->get(exec, (uint32_t)prop);
            }
            break;

            case OpByteCode_BracketGetAndBind_Ireg_Ireg_Rvalue_Rint32: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -16)->regVal;
                Register objDest = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t prop = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 751 "codes.def"
                JSObject* baseObj = base->toObject(exec);
                // toObject may fail and return an exception object, but get from it is harmless
                localStore[objDest].val.valueVal = baseObj;

                localStore[fbDestReg].val.valueVal = baseObj->get(exec, (uint32_t)prop);
            }
            break;

            case OpByteCode_BracketPutKnownObject_Rvalue_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BracketPutKnownObject_Rvalue_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v3 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 762 "codes.def"
                ASSERT(v1->isObject());
                uint32_t i;
                JSObject *o = static_cast<JSObject*>(v1);

                if (v2->getUInt32(i))
                    o->put(exec, i, v3);
                else
                    o->put(exec, Identifier(v2->toString(exec)), v3);
            }
            break;

            case OpByteCode_BracketPutKnownObject_Rvalue_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BracketPutKnownObject_Rvalue_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v3 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 762 "codes.def"
                ASSERT(v1->isObject());
                uint32_t i;
                JSObject *o = static_cast<JSObject*>(v1);

                if (v2->getUInt32(i))
                    o->put(exec, i, v3);
                else
                    o->put(exec, Identifier(v2->toString(exec)), v3);
            }
            break;

            case OpByteCode_BracketPutKnownObject_Rvalue_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BracketPutKnownObject_Rvalue_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v3 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 762 "codes.def"
                ASSERT(v1->isObject());
                uint32_t i;
                JSObject *o = static_cast<JSObject*>(v1);

                if (v2->getUInt32(i))
                    o->put(exec, i, v3);
                else
                    o->put(exec, Identifier(v2->toString(exec)), v3);
            }
            break;

            case OpByteCode_BracketPutKnownObject_Rvalue_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -12)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v3 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 762 "codes.def"
                ASSERT(v1->isObject());
                uint32_t i;
                JSObject *o = static_cast<JSObject*>(v1);

                if (v2->getUInt32(i))
                    o->put(exec, i, v3);
                else
                    o->put(exec, Identifier(v2->toString(exec)), v3);
            }
            break;

            case OpByteCode_BracketPutKnownObject_Rvalue_Iint32_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BracketPutKnownObject_Rvalue_Iint32_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t prop = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
                JSValue* val = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 773 "codes.def"
                ASSERT(base->isObject());
                static_cast<JSObject*>(base)->put(exec, (uint32_t)prop, val);
            }
            break;

            case OpByteCode_BracketPutKnownObject_Rvalue_Iint32_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -12)->regVal)->val.valueVal;
                int32_t prop = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                JSValue* val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 773 "codes.def"
                ASSERT(base->isObject());
                static_cast<JSObject*>(base)->put(exec, (uint32_t)prop, val);
            }
            break;

            case OpByteCode_BracketPutKnownObject_Rvalue_Rint32_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BracketPutKnownObject_Rvalue_Rint32_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t prop = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
                JSValue* val = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 773 "codes.def"
                ASSERT(base->isObject());
                static_cast<JSObject*>(base)->put(exec, (uint32_t)prop, val);
            }
            break;

            case OpByteCode_BracketPutKnownObject_Rvalue_Rint32_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -12)->regVal)->val.valueVal;
                int32_t prop = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                JSValue* val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 773 "codes.def"
                ASSERT(base->isObject());
                static_cast<JSObject*>(base)->put(exec, (uint32_t)prop, val);
            }
            break;

            case OpByteCode_GlobalObjectGet_Ireg_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_GlobalObjectGet_Ireg_Iident: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* varName = reinterpret_cast<const WideArg*>(localPC -12)->identVal;
#line 780 "codes.def"
                JSObject* scopeObj = globalObject;
                PropertySlot slot;
                if (scopeObj->getPropertySlot(exec, *varName, slot)) {
                    JSValue* val = slot.getValue(exec, scopeObj, *varName);;
                    localStore[fbDestReg].val.valueVal = val;
                } else {
                    throwUndefinedVariableError(exec, *varName);
                }
            }
            break;

            case OpByteCode_SymDeleteKnownObject_Ireg_Ivalue_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymDeleteKnownObject_Ireg_Ivalue_Iident: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* scope = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                Identifier* name = reinterpret_cast<const WideArg*>(localPC -12)->identVal;
#line 793 "codes.def"
                ASSERT(scope->isObject());
                localStore[fbDestReg].val.boolVal = static_cast<JSObject*>(scope)->deleteProperty(exec, *name);
            }
            break;

            case OpByteCode_SymDeleteKnownObject_Ireg_Rvalue_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymDeleteKnownObject_Ireg_Rvalue_Iident: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* scope = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                Identifier* name = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
#line 793 "codes.def"
                ASSERT(scope->isObject());
                localStore[fbDestReg].val.boolVal = static_cast<JSObject*>(scope)->deleteProperty(exec, *name);
            }
            break;

            case OpByteCode_SymDelete_Ireg_Ivalue_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymDelete_Ireg_Ivalue_Iident: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* scope = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                Identifier* name = reinterpret_cast<const WideArg*>(localPC -12)->identVal;
#line 800 "codes.def"
                JSObject* o = scope->toObject(exec);
                if (pc == localPC) // No exception!
                    localStore[fbDestReg].val.boolVal = o->deleteProperty(exec, *name);
            }
            break;

            case OpByteCode_SymDelete_Ireg_Rvalue_Iident_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_SymDelete_Ireg_Rvalue_Iident: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* scope = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                Identifier* name = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
#line 800 "codes.def"
                JSObject* o = scope->toObject(exec);
                if (pc == localPC) // No exception!
                    localStore[fbDestReg].val.boolVal = o->deleteProperty(exec, *name);
            }
            break;

            case OpByteCode_BracketDelete_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BracketDelete_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 808 "codes.def"
                uint32_t i;
                JSObject *o = v1->toObject(exec);
                if (pc != localPC)
                    break;

                if (v2->getUInt32(i))
                    localStore[fbDestReg].val.boolVal = o->deleteProperty(exec, i);
                else
                    localStore[fbDestReg].val.boolVal = o->deleteProperty(exec, Identifier(v2->toString(exec)));
            }
            break;

            case OpByteCode_BracketDelete_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 808 "codes.def"
                uint32_t i;
                JSObject *o = v1->toObject(exec);
                if (pc != localPC)
                    break;

                if (v2->getUInt32(i))
                    localStore[fbDestReg].val.boolVal = o->deleteProperty(exec, i);
                else
                    localStore[fbDestReg].val.boolVal = o->deleteProperty(exec, Identifier(v2->toString(exec)));
            }
            break;

            case OpByteCode_BracketDelete_Ireg_Ivalue_Iint32_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BracketDelete_Ireg_Ivalue_Iint32: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                int32_t i = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 820 "codes.def"
                JSObject *o = v1->toObject(exec);
                if (pc != localPC)
                    break;

                localStore[fbDestReg].val.boolVal = o->deleteProperty(exec, (uint32_t)i);
            }
            break;

            case OpByteCode_BracketDelete_Ireg_Ivalue_Rint32_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_BracketDelete_Ireg_Ivalue_Rint32: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                int32_t i = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 820 "codes.def"
                JSObject *o = v1->toObject(exec);
                if (pc != localPC)
                    break;

                localStore[fbDestReg].val.boolVal = o->deleteProperty(exec, (uint32_t)i);
            }
            break;

            case OpByteCode_BracketDelete_Ireg_Rvalue_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t i = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 820 "codes.def"
                JSObject *o = v1->toObject(exec);
                if (pc != localPC)
                    break;

                localStore[fbDestReg].val.boolVal = o->deleteProperty(exec, (uint32_t)i);
            }
            break;

            case OpByteCode_BracketDelete_Ireg_Rvalue_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t i = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 820 "codes.def"
                JSObject *o = v1->toObject(exec);
                if (pc != localPC)
                    break;

                localStore[fbDestReg].val.boolVal = o->deleteProperty(exec, (uint32_t)i);
            }
            break;

            case OpByteCode_OwnedString_Ireg_Istring_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_OwnedString_Ireg_Istring: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                UString* in = reinterpret_cast<const WideArg*>(localPC -12)->stringVal;
#line 833 "codes.def"
                localStore[fbDestReg].val.valueVal = jsOwnedString(*in);
            }
            break;

            case OpByteCode_NewEmptyArray_Ireg: {
                pc += 8;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
#line 839 "codes.def"
                localStore[fbDestReg].val.valueVal = exec->lexicalInterpreter()->builtinArray()->construct(exec, List::empty());
            }
            break;

            case OpByteCode_NewObject_Ireg: {
                pc += 8;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
#line 845 "codes.def"
                localStore[fbDestReg].val.valueVal = new JSObject(exec->lexicalInterpreter()->builtinObjectPrototype());
            }
            break;

            case OpByteCode_NewRegExp_Ireg_Istring_Istring_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NewRegExp_Ireg_Istring_Istring: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                UString* pattern = reinterpret_cast<const WideArg*>(localPC -20)->stringVal;
                UString* flags = reinterpret_cast<const WideArg*>(localPC -12)->stringVal;
#line 851 "codes.def"
                // ### TODO: this is -very- wasteful
                List list;
                list.append(jsOwnedString(*pattern));
                list.append(jsOwnedString(*flags));

                JSObject *reg = exec->lexicalInterpreter()->builtinRegExp();
                localStore[fbDestReg].val.valueVal = reg->construct(exec,list);
            }
            break;

            case OpByteCode_DefineGetter_Rvalue_Iident_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_DefineGetter_Rvalue_Iident_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                Identifier* name = reinterpret_cast<const WideArg*>(localPC -20)->identVal;
                JSValue* getter = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 863 "codes.def"
                ASSERT(base->isObject());
                ASSERT(getter->isObject());
                static_cast<JSObject*>(base)->defineGetter(exec, *name, static_cast<JSObject*>(getter));
            }
            break;

            case OpByteCode_DefineGetter_Rvalue_Iident_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_DefineGetter_Rvalue_Iident_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                Identifier* name = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
                JSValue* getter = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 863 "codes.def"
                ASSERT(base->isObject());
                ASSERT(getter->isObject());
                static_cast<JSObject*>(base)->defineGetter(exec, *name, static_cast<JSObject*>(getter));
            }
            break;

            case OpByteCode_DefineSetter_Rvalue_Iident_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_DefineSetter_Rvalue_Iident_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                Identifier* name = reinterpret_cast<const WideArg*>(localPC -20)->identVal;
                JSValue* setter = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 871 "codes.def"
                ASSERT(base->isObject());
                ASSERT(setter->isObject());
                static_cast<JSObject*>(base)->defineSetter(exec, *name, static_cast<JSObject*>(setter));
            }
            break;

            case OpByteCode_DefineSetter_Rvalue_Iident_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_DefineSetter_Rvalue_Iident_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* base = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                Identifier* name = reinterpret_cast<const WideArg*>(localPC -16)->identVal;
                JSValue* setter = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 871 "codes.def"
                ASSERT(base->isObject());
                ASSERT(setter->isObject());
                static_cast<JSObject*>(base)->defineSetter(exec, *name, static_cast<JSObject*>(setter));
            }
            break;

            case OpByteCode_Add1_Ireg_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add1_Ireg_Inumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double old = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 883 "codes.def"
                localStore[fbDestReg].val.numberVal = old + 1.0;
            }
            break;

            case OpByteCode_Add1_Ireg_Rnumber: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double old = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 883 "codes.def"
                localStore[fbDestReg].val.numberVal = old + 1.0;
            }
            break;

            case OpByteCode_Add1_Ireg_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double old = convertIValue_Number(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
#line 883 "codes.def"
                localStore[fbDestReg].val.numberVal = old + 1.0;
            }
            break;

            case OpByteCode_Sub1_Ireg_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Sub1_Ireg_Inumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double old = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 891 "codes.def"
                localStore[fbDestReg].val.numberVal = old - 1.0;
            }
            break;

            case OpByteCode_Sub1_Ireg_Rnumber: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double old = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 891 "codes.def"
                localStore[fbDestReg].val.numberVal = old - 1.0;
            }
            break;

            case OpByteCode_Neg_Ireg_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Neg_Ireg_Inumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double old = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 897 "codes.def"
                localStore[fbDestReg].val.numberVal = -old;
            }
            break;

            case OpByteCode_Neg_Ireg_Rnumber: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double old = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 897 "codes.def"
                localStore[fbDestReg].val.numberVal = -old;
            }
            break;

            case OpByteCode_Neg_Ireg_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double old = convertIValue_Number(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
#line 897 "codes.def"
                localStore[fbDestReg].val.numberVal = -old;
            }
            break;

            case OpByteCode_Mult_Ireg_Inumber_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Mult_Ireg_Inumber_Inumber: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double v1 = reinterpret_cast<const WideArg*>(localPC -20)->numberVal;
                double v2 = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 905 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 * v2;
            }
            break;

            case OpByteCode_Mult_Ireg_Inumber_Rnumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Mult_Ireg_Inumber_Rnumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double v1 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
                double v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 905 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 * v2;
            }
            break;

            case OpByteCode_Mult_Ireg_Rnumber_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Mult_Ireg_Rnumber_Inumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
                double v2 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
#line 905 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 * v2;
            }
            break;

            case OpByteCode_Mult_Ireg_Rnumber_Rnumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                double v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.numberVal;
                double v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 905 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 * v2;
            }
            break;

            case OpByteCode_Mult_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                double v1 = convertIValue_Number(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
                double v2 = convertIValue_Number(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
#line 905 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 * v2;
            }
            break;

            case OpByteCode_Mult_Ireg_Rvalue_Rnumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                double v1 = convertIValue_Number(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
                double v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 905 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 * v2;
            }
            break;

            case OpByteCode_Mult_Ireg_Rnumber_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                double v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.numberVal;
                double v2 = convertIValue_Number(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
#line 905 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 * v2;
            }
            break;

            case OpByteCode_Div_Ireg_Inumber_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Div_Ireg_Inumber_Inumber: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double v1 = reinterpret_cast<const WideArg*>(localPC -20)->numberVal;
                double v2 = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 915 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 / v2;
            }
            break;

            case OpByteCode_Div_Ireg_Inumber_Rnumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Div_Ireg_Inumber_Rnumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double v1 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
                double v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 915 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 / v2;
            }
            break;

            case OpByteCode_Div_Ireg_Rnumber_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Div_Ireg_Rnumber_Inumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
                double v2 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
#line 915 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 / v2;
            }
            break;

            case OpByteCode_Div_Ireg_Rnumber_Rnumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                double v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.numberVal;
                double v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 915 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 / v2;
            }
            break;

            case OpByteCode_Mod_Ireg_Inumber_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Mod_Ireg_Inumber_Inumber: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double v1 = reinterpret_cast<const WideArg*>(localPC -20)->numberVal;
                double v2 = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 921 "codes.def"
                localStore[fbDestReg].val.numberVal = fmod(v1, v2);
            }
            break;

            case OpByteCode_Mod_Ireg_Inumber_Rnumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Mod_Ireg_Inumber_Rnumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double v1 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
                double v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 921 "codes.def"
                localStore[fbDestReg].val.numberVal = fmod(v1, v2);
            }
            break;

            case OpByteCode_Mod_Ireg_Rnumber_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Mod_Ireg_Rnumber_Inumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
                double v2 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
#line 921 "codes.def"
                localStore[fbDestReg].val.numberVal = fmod(v1, v2);
            }
            break;

            case OpByteCode_Mod_Ireg_Rnumber_Rnumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                double v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.numberVal;
                double v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 921 "codes.def"
                localStore[fbDestReg].val.numberVal = fmod(v1, v2);
            }
            break;

            case OpByteCode_Sub_Ireg_Inumber_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Sub_Ireg_Inumber_Inumber: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double v1 = reinterpret_cast<const WideArg*>(localPC -20)->numberVal;
                double v2 = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 927 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 - v2;
            }
            break;

            case OpByteCode_Sub_Ireg_Inumber_Rnumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Sub_Ireg_Inumber_Rnumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double v1 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
                double v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 927 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 - v2;
            }
            break;

            case OpByteCode_Sub_Ireg_Rnumber_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Sub_Ireg_Rnumber_Inumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
                double v2 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
#line 927 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 - v2;
            }
            break;

            case OpByteCode_Sub_Ireg_Rnumber_Rnumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                double v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.numberVal;
                double v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 927 "codes.def"
                localStore[fbDestReg].val.numberVal = v1 - v2;
            }
            break;

            case OpByteCode_Add_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 933 "codes.def"
                // exception for the Date exception in defaultValue()
                JSValue *p1 = v1->toPrimitive(exec, UnspecifiedType);
                if (pc != localPC) break;
                JSValue *p2 = v2->toPrimitive(exec, UnspecifiedType);
                if (pc != localPC) break;

                if (p1->isString() || p2->isString()) {
                    UString value = p1->toString(exec) + p2->toString(exec);
                    if (value.isNull()) {
                        JSObject *error = Error::create(exec, GeneralError, "Out of memory");
                        exec->setException(error);
                    } else {
                        localStore[fbDestReg].val.valueVal = jsString(value);
                    }
                } else {
                    localStore[fbDestReg].val.valueVal = jsNumber(p1->toNumber(exec) + p2->toNumber(exec));
                }
            }
            break;

            case OpByteCode_Add_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 933 "codes.def"
                // exception for the Date exception in defaultValue()
                JSValue *p1 = v1->toPrimitive(exec, UnspecifiedType);
                if (pc != localPC) break;
                JSValue *p2 = v2->toPrimitive(exec, UnspecifiedType);
                if (pc != localPC) break;

                if (p1->isString() || p2->isString()) {
                    UString value = p1->toString(exec) + p2->toString(exec);
                    if (value.isNull()) {
                        JSObject *error = Error::create(exec, GeneralError, "Out of memory");
                        exec->setException(error);
                    } else {
                        localStore[fbDestReg].val.valueVal = jsString(value);
                    }
                } else {
                    localStore[fbDestReg].val.valueVal = jsNumber(p1->toNumber(exec) + p2->toNumber(exec));
                }
            }
            break;

            case OpByteCode_Add_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 933 "codes.def"
                // exception for the Date exception in defaultValue()
                JSValue *p1 = v1->toPrimitive(exec, UnspecifiedType);
                if (pc != localPC) break;
                JSValue *p2 = v2->toPrimitive(exec, UnspecifiedType);
                if (pc != localPC) break;

                if (p1->isString() || p2->isString()) {
                    UString value = p1->toString(exec) + p2->toString(exec);
                    if (value.isNull()) {
                        JSObject *error = Error::create(exec, GeneralError, "Out of memory");
                        exec->setException(error);
                    } else {
                        localStore[fbDestReg].val.valueVal = jsString(value);
                    }
                } else {
                    localStore[fbDestReg].val.valueVal = jsNumber(p1->toNumber(exec) + p2->toNumber(exec));
                }
            }
            break;

            case OpByteCode_Add_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 933 "codes.def"
                // exception for the Date exception in defaultValue()
                JSValue *p1 = v1->toPrimitive(exec, UnspecifiedType);
                if (pc != localPC) break;
                JSValue *p2 = v2->toPrimitive(exec, UnspecifiedType);
                if (pc != localPC) break;

                if (p1->isString() || p2->isString()) {
                    UString value = p1->toString(exec) + p2->toString(exec);
                    if (value.isNull()) {
                        JSObject *error = Error::create(exec, GeneralError, "Out of memory");
                        exec->setException(error);
                    } else {
                        localStore[fbDestReg].val.valueVal = jsString(value);
                    }
                } else {
                    localStore[fbDestReg].val.valueVal = jsNumber(p1->toNumber(exec) + p2->toNumber(exec));
                }
            }
            break;

            case OpByteCode_Add_Ireg_Inumber_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add_Ireg_Inumber_Inumber: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                double n1 = reinterpret_cast<const WideArg*>(localPC -20)->numberVal;
                double n2 = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 953 "codes.def"
                localStore[fbDestReg].val.numberVal = n1 + n2;
            }
            break;

            case OpByteCode_Add_Ireg_Inumber_Rnumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add_Ireg_Inumber_Rnumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double n1 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
                double n2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 953 "codes.def"
                localStore[fbDestReg].val.numberVal = n1 + n2;
            }
            break;

            case OpByteCode_Add_Ireg_Rnumber_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add_Ireg_Rnumber_Inumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                double n1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
                double n2 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
#line 953 "codes.def"
                localStore[fbDestReg].val.numberVal = n1 + n2;
            }
            break;

            case OpByteCode_Add_Ireg_Rnumber_Rnumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                double n1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.numberVal;
                double n2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 953 "codes.def"
                localStore[fbDestReg].val.numberVal = n1 + n2;
            }
            break;

            case OpByteCode_LShift_Ireg_Iint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t v2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 960 "codes.def"
                localStore[fbDestReg].val.int32Val = (v1 << ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_LShift_Ireg_Iint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 960 "codes.def"
                localStore[fbDestReg].val.int32Val = (v1 << ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_LShift_Ireg_Rint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t v2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 960 "codes.def"
                localStore[fbDestReg].val.int32Val = (v1 << ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_LShift_Ireg_Rint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 960 "codes.def"
                localStore[fbDestReg].val.int32Val = (v1 << ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_RShift_Ireg_Iint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t v2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 966 "codes.def"
                localStore[fbDestReg].val.int32Val = (v1 >> ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_RShift_Ireg_Iint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 966 "codes.def"
                localStore[fbDestReg].val.int32Val = (v1 >> ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_RShift_Ireg_Rint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t v2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 966 "codes.def"
                localStore[fbDestReg].val.int32Val = (v1 >> ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_RShift_Ireg_Rint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 966 "codes.def"
                localStore[fbDestReg].val.int32Val = (v1 >> ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_URShift_Ireg_Iint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t v2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 973 "codes.def"
                localStore[fbDestReg].val.numberVal = (uint32_t(v1) >> ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_URShift_Ireg_Iint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 973 "codes.def"
                localStore[fbDestReg].val.numberVal = (uint32_t(v1) >> ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_URShift_Ireg_Rint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t v2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 973 "codes.def"
                localStore[fbDestReg].val.numberVal = (uint32_t(v1) >> ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_URShift_Ireg_Rint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 973 "codes.def"
                localStore[fbDestReg].val.numberVal = (uint32_t(v1) >> ((uint32_t)v2 & 0x1f));
            }
            break;

            case OpByteCode_Less_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Less_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 981 "codes.def"
                // operator <
                // -1: NaN, undefined => false
                //  0: v1 >= v2       => false
                //  1: v1 <  v2       => true;
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, v2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Less_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 981 "codes.def"
                // operator <
                // -1: NaN, undefined => false
                //  0: v1 >= v2       => false
                //  1: v1 <  v2       => true;
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, v2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Less_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 981 "codes.def"
                // operator <
                // -1: NaN, undefined => false
                //  0: v1 >= v2       => false
                //  1: v1 <  v2       => true;
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, v2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 981 "codes.def"
                // operator <
                // -1: NaN, undefined => false
                //  0: v1 >= v2       => false
                //  1: v1 <  v2       => true;
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, v2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Ivalue_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Less_Ireg_Ivalue_Inumber: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                double n2 = reinterpret_cast<const WideArg*>(localPC -12)->numberVal;
#line 989 "codes.def"
                // As above..
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, n2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Ivalue_Rnumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Less_Ireg_Ivalue_Rnumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                double n2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 989 "codes.def"
                // As above..
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, n2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Rvalue_Inumber_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Less_Ireg_Rvalue_Inumber: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                double n2 = reinterpret_cast<const WideArg*>(localPC -16)->numberVal;
#line 989 "codes.def"
                // As above..
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, n2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Rvalue_Rnumber: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                double n2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.numberVal;
#line 989 "codes.def"
                // As above..
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, n2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Ivalue_Iint32_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Less_Ireg_Ivalue_Iint32: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                int32_t n2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 994 "codes.def"
                // As above... 
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, n2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Ivalue_Rint32_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Less_Ireg_Ivalue_Rint32: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                int32_t n2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 994 "codes.def"
                // As above... 
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, n2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Rvalue_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t n2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 994 "codes.def"
                // As above... 
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, n2) == 1);
            }
            break;

            case OpByteCode_Less_Ireg_Rvalue_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t n2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 994 "codes.def"
                // As above... 
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, n2) == 1);
            }
            break;

            case OpByteCode_GreaterEq_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_GreaterEq_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1001 "codes.def"
                // operator >=
                // -1: NaN, undefined => false
                //  0: v1 >= v2       => true
                //  1: v1 <  v2       => false;
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, v2) == 0);
            }
            break;

            case OpByteCode_GreaterEq_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_GreaterEq_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1001 "codes.def"
                // operator >=
                // -1: NaN, undefined => false
                //  0: v1 >= v2       => true
                //  1: v1 <  v2       => false;
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, v2) == 0);
            }
            break;

            case OpByteCode_GreaterEq_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_GreaterEq_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1001 "codes.def"
                // operator >=
                // -1: NaN, undefined => false
                //  0: v1 >= v2       => true
                //  1: v1 <  v2       => false;
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, v2) == 0);
            }
            break;

            case OpByteCode_GreaterEq_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1001 "codes.def"
                // operator >=
                // -1: NaN, undefined => false
                //  0: v1 >= v2       => true
                //  1: v1 <  v2       => false;
                localStore[fbDestReg].val.boolVal = (relation(exec, v1, v2) == 0);
            }
            break;

            case OpByteCode_Greater_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Greater_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1011 "codes.def"
                // operator >
                // -1: NaN, undefined => false
                //  0: v2 >= v1       => false
                //  1: v2 <  v1       => true
                localStore[fbDestReg].val.boolVal = (relation(exec, v2, v1) == 1);
            }
            break;

            case OpByteCode_Greater_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Greater_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1011 "codes.def"
                // operator >
                // -1: NaN, undefined => false
                //  0: v2 >= v1       => false
                //  1: v2 <  v1       => true
                localStore[fbDestReg].val.boolVal = (relation(exec, v2, v1) == 1);
            }
            break;

            case OpByteCode_Greater_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Greater_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1011 "codes.def"
                // operator >
                // -1: NaN, undefined => false
                //  0: v2 >= v1       => false
                //  1: v2 <  v1       => true
                localStore[fbDestReg].val.boolVal = (relation(exec, v2, v1) == 1);
            }
            break;

            case OpByteCode_Greater_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1011 "codes.def"
                // operator >
                // -1: NaN, undefined => false
                //  0: v2 >= v1       => false
                //  1: v2 <  v1       => true
                localStore[fbDestReg].val.boolVal = (relation(exec, v2, v1) == 1);
            }
            break;

            case OpByteCode_LessEq_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_LessEq_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1021 "codes.def"
                // operator <=
                // -1: NaN, undefined => false
                //  0: v2 >= v1       => true
                //  1: v2 <  v1       => false
                localStore[fbDestReg].val.boolVal = (relation(exec, v2, v1) == 0);
            }
            break;

            case OpByteCode_LessEq_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_LessEq_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1021 "codes.def"
                // operator <=
                // -1: NaN, undefined => false
                //  0: v2 >= v1       => true
                //  1: v2 <  v1       => false
                localStore[fbDestReg].val.boolVal = (relation(exec, v2, v1) == 0);
            }
            break;

            case OpByteCode_LessEq_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_LessEq_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1021 "codes.def"
                // operator <=
                // -1: NaN, undefined => false
                //  0: v2 >= v1       => true
                //  1: v2 <  v1       => false
                localStore[fbDestReg].val.boolVal = (relation(exec, v2, v1) == 0);
            }
            break;

            case OpByteCode_LessEq_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1021 "codes.def"
                // operator <=
                // -1: NaN, undefined => false
                //  0: v2 >= v1       => true
                //  1: v2 <  v1       => false
                localStore[fbDestReg].val.boolVal = (relation(exec, v2, v1) == 0);
            }
            break;

            case OpByteCode_EqEq_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_EqEq_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1031 "codes.def"
                // operator ==
                localStore[fbDestReg].val.boolVal = equal(exec, v1, v2);
            }
            break;

            case OpByteCode_EqEq_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_EqEq_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1031 "codes.def"
                // operator ==
                localStore[fbDestReg].val.boolVal = equal(exec, v1, v2);
            }
            break;

            case OpByteCode_EqEq_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_EqEq_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1031 "codes.def"
                // operator ==
                localStore[fbDestReg].val.boolVal = equal(exec, v1, v2);
            }
            break;

            case OpByteCode_EqEq_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1031 "codes.def"
                // operator ==
                localStore[fbDestReg].val.boolVal = equal(exec, v1, v2);
            }
            break;

            case OpByteCode_EqEq_Ireg_Ivalue_Iint32_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_EqEq_Ireg_Ivalue_Iint32: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                int32_t n2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1036 "codes.def"
                double n1;
                if (v1->getNumber(n1))
                    localStore[fbDestReg].val.boolVal = (n1 == n2);
                else
                    localStore[fbDestReg].val.boolVal = equal(exec, v1, jsNumber(n2));
            }
            break;

            case OpByteCode_EqEq_Ireg_Ivalue_Rint32_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_EqEq_Ireg_Ivalue_Rint32: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                int32_t n2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1036 "codes.def"
                double n1;
                if (v1->getNumber(n1))
                    localStore[fbDestReg].val.boolVal = (n1 == n2);
                else
                    localStore[fbDestReg].val.boolVal = equal(exec, v1, jsNumber(n2));
            }
            break;

            case OpByteCode_EqEq_Ireg_Rvalue_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t n2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1036 "codes.def"
                double n1;
                if (v1->getNumber(n1))
                    localStore[fbDestReg].val.boolVal = (n1 == n2);
                else
                    localStore[fbDestReg].val.boolVal = equal(exec, v1, jsNumber(n2));
            }
            break;

            case OpByteCode_EqEq_Ireg_Rvalue_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t n2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1036 "codes.def"
                double n1;
                if (v1->getNumber(n1))
                    localStore[fbDestReg].val.boolVal = (n1 == n2);
                else
                    localStore[fbDestReg].val.boolVal = equal(exec, v1, jsNumber(n2));
            }
            break;

            case OpByteCode_NotEq_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NotEq_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1046 "codes.def"
                // operator !=
                localStore[fbDestReg].val.boolVal = !equal(exec,v1, v2);
            }
            break;

            case OpByteCode_NotEq_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NotEq_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1046 "codes.def"
                // operator !=
                localStore[fbDestReg].val.boolVal = !equal(exec,v1, v2);
            }
            break;

            case OpByteCode_NotEq_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NotEq_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1046 "codes.def"
                // operator !=
                localStore[fbDestReg].val.boolVal = !equal(exec,v1, v2);
            }
            break;

            case OpByteCode_NotEq_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1046 "codes.def"
                // operator !=
                localStore[fbDestReg].val.boolVal = !equal(exec,v1, v2);
            }
            break;

            case OpByteCode_NotEq_Ireg_Ivalue_Iint32_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NotEq_Ireg_Ivalue_Iint32: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                int32_t n2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1051 "codes.def"
                double n1;
                if (v1->getNumber(n1))
                    localStore[fbDestReg].val.boolVal = (n1 != n2);
                else
                    localStore[fbDestReg].val.boolVal = !equal(exec, v1, jsNumber(n2));
            }
            break;

            case OpByteCode_NotEq_Ireg_Ivalue_Rint32_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_NotEq_Ireg_Ivalue_Rint32: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                int32_t n2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1051 "codes.def"
                double n1;
                if (v1->getNumber(n1))
                    localStore[fbDestReg].val.boolVal = (n1 != n2);
                else
                    localStore[fbDestReg].val.boolVal = !equal(exec, v1, jsNumber(n2));
            }
            break;

            case OpByteCode_NotEq_Ireg_Rvalue_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t n2 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1051 "codes.def"
                double n1;
                if (v1->getNumber(n1))
                    localStore[fbDestReg].val.boolVal = (n1 != n2);
                else
                    localStore[fbDestReg].val.boolVal = !equal(exec, v1, jsNumber(n2));
            }
            break;

            case OpByteCode_NotEq_Ireg_Rvalue_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                int32_t n2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1051 "codes.def"
                double n1;
                if (v1->getNumber(n1))
                    localStore[fbDestReg].val.boolVal = (n1 != n2);
                else
                    localStore[fbDestReg].val.boolVal = !equal(exec, v1, jsNumber(n2));
            }
            break;

            case OpByteCode_StrEq_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_StrEq_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1062 "codes.def"
                // operator ===
                localStore[fbDestReg].val.boolVal = strictEqual(exec,v1, v2);
            }
            break;

            case OpByteCode_StrEq_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_StrEq_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1062 "codes.def"
                // operator ===
                localStore[fbDestReg].val.boolVal = strictEqual(exec,v1, v2);
            }
            break;

            case OpByteCode_StrEq_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_StrEq_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1062 "codes.def"
                // operator ===
                localStore[fbDestReg].val.boolVal = strictEqual(exec,v1, v2);
            }
            break;

            case OpByteCode_StrEq_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1062 "codes.def"
                // operator ===
                localStore[fbDestReg].val.boolVal = strictEqual(exec,v1, v2);
            }
            break;

            case OpByteCode_StrNEq_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_StrNEq_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1069 "codes.def"
                // operator !==
                localStore[fbDestReg].val.boolVal = !strictEqual(exec,v1, v2);
            }
            break;

            case OpByteCode_StrNEq_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_StrNEq_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1069 "codes.def"
                // operator !==
                localStore[fbDestReg].val.boolVal = !strictEqual(exec,v1, v2);
            }
            break;

            case OpByteCode_StrNEq_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_StrNEq_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1069 "codes.def"
                // operator !==
                localStore[fbDestReg].val.boolVal = !strictEqual(exec,v1, v2);
            }
            break;

            case OpByteCode_StrNEq_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1069 "codes.def"
                // operator !==
                localStore[fbDestReg].val.boolVal = !strictEqual(exec,v1, v2);
            }
            break;

            case OpByteCode_BitNot_Ireg_Iint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t old = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1080 "codes.def"
                localStore[fbDestReg].val.int32Val = ~old;
            }
            break;

            case OpByteCode_BitNot_Ireg_Rint32: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                int32_t old = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1080 "codes.def"
                localStore[fbDestReg].val.int32Val = ~old;
            }
            break;

            case OpByteCode_BitAnd_Ireg_Iint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t a1 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1086 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 & a1;
            }
            break;

            case OpByteCode_BitAnd_Ireg_Iint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1086 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 & a1;
            }
            break;

            case OpByteCode_BitAnd_Ireg_Rint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t a1 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1086 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 & a1;
            }
            break;

            case OpByteCode_BitAnd_Ireg_Rint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1086 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 & a1;
            }
            break;

            case OpByteCode_BitAnd_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = convertIValue_Int32(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
                int32_t a1 = convertIValue_Int32(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
#line 1086 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 & a1;
            }
            break;

            case OpByteCode_BitAnd_Ireg_Rvalue_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = convertIValue_Int32(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
                int32_t a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1086 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 & a1;
            }
            break;

            case OpByteCode_BitAnd_Ireg_Rint32_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t a1 = convertIValue_Int32(exec, reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal);
                if (pc != localPC) // {// Exception or reload
                    continue;
#line 1086 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 & a1;
            }
            break;

            case OpByteCode_BitXOr_Ireg_Iint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t a1 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1096 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 ^ a1;
            }
            break;

            case OpByteCode_BitXOr_Ireg_Iint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1096 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 ^ a1;
            }
            break;

            case OpByteCode_BitXOr_Ireg_Rint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t a1 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1096 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 ^ a1;
            }
            break;

            case OpByteCode_BitXOr_Ireg_Rint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1096 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 ^ a1;
            }
            break;

            case OpByteCode_BitOr_Ireg_Iint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t a1 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1102 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 | a1;
            }
            break;

            case OpByteCode_BitOr_Ireg_Iint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const NarrowArg*>(localPC -8)->int32Val;
                int32_t a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1102 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 | a1;
            }
            break;

            case OpByteCode_BitOr_Ireg_Rint32_Iint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t a1 = reinterpret_cast<const NarrowArg*>(localPC -4)->int32Val;
#line 1102 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 | a1;
            }
            break;

            case OpByteCode_BitOr_Ireg_Rint32_Rint32: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                int32_t a0 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.int32Val;
                int32_t a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.int32Val;
#line 1102 "codes.def"
                localStore[fbDestReg].val.int32Val = a0 | a1;
            }
            break;

            case OpByteCode_LogicalNot_Ireg_Ibool: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                bool old = reinterpret_cast<const NarrowArg*>(localPC -4)->boolVal;
#line 1112 "codes.def"
                localStore[fbDestReg].val.boolVal = !old;
            }
            break;

            case OpByteCode_LogicalNot_Ireg_Rbool: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                bool old = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.boolVal;
#line 1112 "codes.def"
                localStore[fbDestReg].val.boolVal = !old;
            }
            break;

            case OpByteCode_TypeOf_Ireg_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_TypeOf_Ireg_Ivalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1121 "codes.def"
                localStore[fbDestReg].val.valueVal = typeStringForValue(v);
            }
            break;

            case OpByteCode_TypeOf_Ireg_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1121 "codes.def"
                localStore[fbDestReg].val.valueVal = typeStringForValue(v);
            }
            break;

            case OpByteCode_In_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_In_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1127 "codes.def"
                if (!v2->isObject()) {
                    throwError(exec,  TypeError, "Value is not an object. Cannot be used with IN expression.");
                    break;
                }
                JSObject* o2 = static_cast<JSObject*>(v2);
                localStore[fbDestReg].val.boolVal = o2->hasProperty(exec, Identifier(v1->toString(exec)));
            }
            break;

            case OpByteCode_In_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_In_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1127 "codes.def"
                if (!v2->isObject()) {
                    throwError(exec,  TypeError, "Value is not an object. Cannot be used with IN expression.");
                    break;
                }
                JSObject* o2 = static_cast<JSObject*>(v2);
                localStore[fbDestReg].val.boolVal = o2->hasProperty(exec, Identifier(v1->toString(exec)));
            }
            break;

            case OpByteCode_In_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_In_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1127 "codes.def"
                if (!v2->isObject()) {
                    throwError(exec,  TypeError, "Value is not an object. Cannot be used with IN expression.");
                    break;
                }
                JSObject* o2 = static_cast<JSObject*>(v2);
                localStore[fbDestReg].val.boolVal = o2->hasProperty(exec, Identifier(v1->toString(exec)));
            }
            break;

            case OpByteCode_In_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1127 "codes.def"
                if (!v2->isObject()) {
                    throwError(exec,  TypeError, "Value is not an object. Cannot be used with IN expression.");
                    break;
                }
                JSObject* o2 = static_cast<JSObject*>(v2);
                localStore[fbDestReg].val.boolVal = o2->hasProperty(exec, Identifier(v1->toString(exec)));
            }
            break;

            case OpByteCode_InstanceOf_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_InstanceOf_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1138 "codes.def"
                if (!v2->isObject()) {
                    throwError(exec,  TypeError, "Value is not an object. Cannot be used with instanceof operator.");
                    break;
                }

                JSObject* o2 = static_cast<JSObject*>(v2);
                if (!o2->implementsHasInstance())
                    // According to the spec, only some types of objects "implement" the [HasInstance] property.
                    // But we are supposed to throw an exception where the object does not "have" the [HasInstance]
                    // property. It seems that all object have the property, but not all implement it, so in this
                    // case we return false (consistent with mozilla)
                    localStore[fbDestReg].val.boolVal = false;
                else
                    localStore[fbDestReg].val.boolVal = o2->hasInstance(exec, v1);
            }
            break;

            case OpByteCode_InstanceOf_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_InstanceOf_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1138 "codes.def"
                if (!v2->isObject()) {
                    throwError(exec,  TypeError, "Value is not an object. Cannot be used with instanceof operator.");
                    break;
                }

                JSObject* o2 = static_cast<JSObject*>(v2);
                if (!o2->implementsHasInstance())
                    // According to the spec, only some types of objects "implement" the [HasInstance] property.
                    // But we are supposed to throw an exception where the object does not "have" the [HasInstance]
                    // property. It seems that all object have the property, but not all implement it, so in this
                    // case we return false (consistent with mozilla)
                    localStore[fbDestReg].val.boolVal = false;
                else
                    localStore[fbDestReg].val.boolVal = o2->hasInstance(exec, v1);
            }
            break;

            case OpByteCode_InstanceOf_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_InstanceOf_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1138 "codes.def"
                if (!v2->isObject()) {
                    throwError(exec,  TypeError, "Value is not an object. Cannot be used with instanceof operator.");
                    break;
                }

                JSObject* o2 = static_cast<JSObject*>(v2);
                if (!o2->implementsHasInstance())
                    // According to the spec, only some types of objects "implement" the [HasInstance] property.
                    // But we are supposed to throw an exception where the object does not "have" the [HasInstance]
                    // property. It seems that all object have the property, but not all implement it, so in this
                    // case we return false (consistent with mozilla)
                    localStore[fbDestReg].val.boolVal = false;
                else
                    localStore[fbDestReg].val.boolVal = o2->hasInstance(exec, v1);
            }
            break;

            case OpByteCode_InstanceOf_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* v2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1138 "codes.def"
                if (!v2->isObject()) {
                    throwError(exec,  TypeError, "Value is not an object. Cannot be used with instanceof operator.");
                    break;
                }

                JSObject* o2 = static_cast<JSObject*>(v2);
                if (!o2->implementsHasInstance())
                    // According to the spec, only some types of objects "implement" the [HasInstance] property.
                    // But we are supposed to throw an exception where the object does not "have" the [HasInstance]
                    // property. It seems that all object have the property, but not all implement it, so in this
                    // case we return false (consistent with mozilla)
                    localStore[fbDestReg].val.boolVal = false;
                else
                    localStore[fbDestReg].val.boolVal = o2->hasInstance(exec, v1);
            }
            break;

            case OpByteCode_ClearArgs: {
                pc += 4;
                const unsigned char* localPC = pc;
#line 1161 "codes.def"
                workList.clear();
            }
            break;

            case OpByteCode_AddArg_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_AddArg_Ivalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                JSValue* val = reinterpret_cast<const WideArg*>(localPC -8)->valueVal;
#line 1167 "codes.def"
                workList.append(val);
            }
            break;

            case OpByteCode_AddArg_Rvalue: {
                pc += 8;
                const unsigned char* localPC = pc;
                JSValue* val = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1167 "codes.def"
                workList.append(val);
            }
            break;

            case OpByteCode_Add2Arg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add2Arg_Ivalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* a2 = reinterpret_cast<const WideArg*>(localPC -8)->valueVal;
#line 1173 "codes.def"
                workList.append(a1);
                workList.append(a2);
            }
            break;

            case OpByteCode_Add2Arg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add2Arg_Ivalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
                JSValue* a2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1173 "codes.def"
                workList.append(a1);
                workList.append(a2);
            }
            break;

            case OpByteCode_Add2Arg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add2Arg_Rvalue_Ivalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* a2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1173 "codes.def"
                workList.append(a1);
                workList.append(a2);
            }
            break;

            case OpByteCode_Add2Arg_Rvalue_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* a2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1173 "codes.def"
                workList.append(a1);
                workList.append(a2);
            }
            break;

            case OpByteCode_Add3Arg_Ivalue_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add3Arg_Ivalue_Ivalue_Ivalue: {
                pc += 28;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const WideArg*>(localPC -24)->valueVal;
                JSValue* a2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* a3 = reinterpret_cast<const WideArg*>(localPC -8)->valueVal;
#line 1180 "codes.def"
                workList.append(a1);
                workList.append(a2);
                workList.append(a3);
            }
            break;

            case OpByteCode_Add3Arg_Ivalue_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add3Arg_Ivalue_Ivalue_Rvalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* a2 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
                JSValue* a3 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1180 "codes.def"
                workList.append(a1);
                workList.append(a2);
                workList.append(a3);
            }
            break;

            case OpByteCode_Add3Arg_Ivalue_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add3Arg_Ivalue_Rvalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* a2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* a3 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1180 "codes.def"
                workList.append(a1);
                workList.append(a2);
                workList.append(a3);
            }
            break;

            case OpByteCode_Add3Arg_Ivalue_Rvalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add3Arg_Ivalue_Rvalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* a2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* a3 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1180 "codes.def"
                workList.append(a1);
                workList.append(a2);
                workList.append(a3);
            }
            break;

            case OpByteCode_Add3Arg_Rvalue_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add3Arg_Rvalue_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* a2 = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* a3 = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1180 "codes.def"
                workList.append(a1);
                workList.append(a2);
                workList.append(a3);
            }
            break;

            case OpByteCode_Add3Arg_Rvalue_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add3Arg_Rvalue_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* a2 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* a3 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1180 "codes.def"
                workList.append(a1);
                workList.append(a2);
                workList.append(a3);
            }
            break;

            case OpByteCode_Add3Arg_Rvalue_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_Add3Arg_Rvalue_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* a2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* a3 = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1180 "codes.def"
                workList.append(a1);
                workList.append(a2);
                workList.append(a3);
            }
            break;

            case OpByteCode_Add3Arg_Rvalue_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                JSValue* a1 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -12)->regVal)->val.valueVal;
                JSValue* a2 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* a3 = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1180 "codes.def"
                workList.append(a1);
                workList.append(a2);
                workList.append(a3);
            }
            break;

            case OpByteCode_FunctionCall_Ireg_Ivalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_FunctionCall_Ireg_Ivalue_Ivalue: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v = reinterpret_cast<const WideArg*>(localPC -20)->valueVal;
                JSValue* thisVal = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1188 "codes.def"
                ASSERT(thisVal->isObject());
                ASSERT(!static_cast<JSObject*>(thisVal)->isActivation());

                if (!v->isObject()) {
                    throwError(exec, TypeError, "Attempt to use a non-object as a function.");
                    //return throwError(exec, TypeError, notAnObjectMessage(), v, expr.get());
                    break;
                }

                JSObject *func = static_cast<JSObject*>(v);

                if (!func->implementsCall()) {
                    throwError(exec, TypeError, "Attempt to use a non-function object as a function.");
                    //return throwError(exec, TypeError, notCallableMessage(), v, expr.get());
                    break;
                }

                JSValue* val;
                val = static_cast<JSObject*>(func)->call(exec, static_cast<JSObject*>(thisVal), workList);
                localStore[fbDestReg].val.valueVal  = val;
            }
            break;

            case OpByteCode_FunctionCall_Ireg_Ivalue_Rvalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_FunctionCall_Ireg_Ivalue_Rvalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
                JSValue* thisVal = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1188 "codes.def"
                ASSERT(thisVal->isObject());
                ASSERT(!static_cast<JSObject*>(thisVal)->isActivation());

                if (!v->isObject()) {
                    throwError(exec, TypeError, "Attempt to use a non-object as a function.");
                    //return throwError(exec, TypeError, notAnObjectMessage(), v, expr.get());
                    break;
                }

                JSObject *func = static_cast<JSObject*>(v);

                if (!func->implementsCall()) {
                    throwError(exec, TypeError, "Attempt to use a non-function object as a function.");
                    //return throwError(exec, TypeError, notCallableMessage(), v, expr.get());
                    break;
                }

                JSValue* val;
                val = static_cast<JSObject*>(func)->call(exec, static_cast<JSObject*>(thisVal), workList);
                localStore[fbDestReg].val.valueVal  = val;
            }
            break;

            case OpByteCode_FunctionCall_Ireg_Rvalue_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_FunctionCall_Ireg_Rvalue_Ivalue: {
                pc += 20;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
                JSValue* thisVal = reinterpret_cast<const WideArg*>(localPC -16)->valueVal;
#line 1188 "codes.def"
                ASSERT(thisVal->isObject());
                ASSERT(!static_cast<JSObject*>(thisVal)->isActivation());

                if (!v->isObject()) {
                    throwError(exec, TypeError, "Attempt to use a non-object as a function.");
                    //return throwError(exec, TypeError, notAnObjectMessage(), v, expr.get());
                    break;
                }

                JSObject *func = static_cast<JSObject*>(v);

                if (!func->implementsCall()) {
                    throwError(exec, TypeError, "Attempt to use a non-function object as a function.");
                    //return throwError(exec, TypeError, notCallableMessage(), v, expr.get());
                    break;
                }

                JSValue* val;
                val = static_cast<JSObject*>(func)->call(exec, static_cast<JSObject*>(thisVal), workList);
                localStore[fbDestReg].val.valueVal  = val;
            }
            break;

            case OpByteCode_FunctionCall_Ireg_Rvalue_Rvalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -12)->regVal;
                JSValue* v = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -8)->regVal)->val.valueVal;
                JSValue* thisVal = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1188 "codes.def"
                ASSERT(thisVal->isObject());
                ASSERT(!static_cast<JSObject*>(thisVal)->isActivation());

                if (!v->isObject()) {
                    throwError(exec, TypeError, "Attempt to use a non-object as a function.");
                    //return throwError(exec, TypeError, notAnObjectMessage(), v, expr.get());
                    break;
                }

                JSObject *func = static_cast<JSObject*>(v);

                if (!func->implementsCall()) {
                    throwError(exec, TypeError, "Attempt to use a non-function object as a function.");
                    //return throwError(exec, TypeError, notCallableMessage(), v, expr.get());
                    break;
                }

                JSValue* val;
                val = static_cast<JSObject*>(func)->call(exec, static_cast<JSObject*>(thisVal), workList);
                localStore[fbDestReg].val.valueVal  = val;
            }
            break;

            case OpByteCode_CtorCall_Ireg_Ivalue_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_CtorCall_Ireg_Ivalue: {
                pc += 16;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                JSValue* v = reinterpret_cast<const WideArg*>(localPC -12)->valueVal;
#line 1213 "codes.def"
                if (!v->isObject()) {
                    throwError(exec, TypeError, "Value is not an object. Cannot be used with new.");
                    //return throwError(exec, TypeError, "Value %s (result of expression %s) is not an object. Cannot be used with new.", v, expr.get());
                    break;
                }

                JSObject *constr = static_cast<JSObject*>(v);
                if (!constr->implementsConstruct()) {
                    throwError(exec, TypeError, "Value is not a constructor. Cannot be used with new.");
                    // ### TODO
                    // return throwError(exec, TypeError, "Value %s (result of expression %s) is not a constructor. Cannot be used with new.", v, expr.get());
                    break;
                }

                JSValue* val = constr->construct(exec, workList);
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_CtorCall_Ireg_Rvalue: {
                pc += 12;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -8)->regVal;
                JSValue* v = reinterpret_cast<const LocalStorageEntry*>(reinterpret_cast<const unsigned char*>(localStore) + reinterpret_cast<const NarrowArg*>(localPC -4)->regVal)->val.valueVal;
#line 1213 "codes.def"
                if (!v->isObject()) {
                    throwError(exec, TypeError, "Value is not an object. Cannot be used with new.");
                    //return throwError(exec, TypeError, "Value %s (result of expression %s) is not an object. Cannot be used with new.", v, expr.get());
                    break;
                }

                JSObject *constr = static_cast<JSObject*>(v);
                if (!constr->implementsConstruct()) {
                    throwError(exec, TypeError, "Value is not a constructor. Cannot be used with new.");
                    // ### TODO
                    // return throwError(exec, TypeError, "Value %s (result of expression %s) is not a constructor. Cannot be used with new.", v, expr.get());
                    break;
                }

                JSValue* val = constr->construct(exec, workList);
                localStore[fbDestReg].val.valueVal = val;
            }
            break;

            case OpByteCode_EvalFuncExpr_Ireg_Iident_Inode_Pad:
                pc += 4;
                // Fallthrough
            case OpByteCode_EvalFuncExpr_Ireg_Iident_Inode: {
                pc += 24;
                const unsigned char* localPC = pc;
                Register fbDestReg = reinterpret_cast<const NarrowArg*>(localPC -4)->regVal;
                Identifier* ident = reinterpret_cast<const WideArg*>(localPC -20)->identVal;
                Node* body = reinterpret_cast<const WideArg*>(localPC -12)->nodeVal;
#line 1234 "codes.def"
                bool named = !ident->isNull();
                JSObject *functionScopeObject = 0;

                if (named) {
                    // named FunctionExpressions can recursively call themselves,
                    // but they won't register with the current scope chain and should
                    // be contained as single property in an anonymous object.
                    functionScopeObject = new JSObject;
                    exec->pushScope(functionScopeObject);
                }

                FunctionImp *func = new FunctionImp(exec, *ident, static_cast<FunctionBodyNode*>(body), exec->scopeChain());
                JSObject *proto = exec->lexicalInterpreter()->builtinObject()->construct(exec, List::empty());
                proto->put(exec, exec->propertyNames().constructor, func, DontEnum);
                func->put(exec, exec->propertyNames().prototype, proto, Internal|DontDelete);

                if (named) {
                    functionScopeObject->put(exec, *ident, func, Internal | ReadOnly | (exec->codeType() == EvalCode ? 0 : DontDelete));
                        exec->popScope();
                }

                localStore[fbDestReg].val.valueVal = func;
            }
            break;

        }

#line 165 "/builddir/build/BUILD/kdelibs-4.1.2/kjs/bytecode/machine.cpp.in"
    }
}

}

// kate: indent-width 4; replace-tabs on; tab-width 4; space-indent on; hl c++;

